@@ -81,6 +81,9 @@ map = M.mapM
 zipWith :: Monad m => (a -> b -> m c) -> [a] -> [b] -> m [c]
 zipWith = M.zipWithM
 
+chain :: (Monad m) => [a -> m a] -> (a -> m a)
+chain = foldr (flip (<=<)) return
+
 filter :: Monad m => (a -> m Bool) -> [a] -> m [a]
 filter = M.filterM
 
@@ -109,6 +112,37 @@ lift5 ::
    m a -> m b -> m c -> m d -> m e -> m r
 lift5 = M.liftM5
 
+
+{-
+that's just (=<<)
+
+liftJoin :: (Monad m) => (a -> m b) -> m a -> m b
+liftJoin f ma =
+   join (lift f ma)
+-}
+
+liftJoin2 :: (Monad m) => (a -> b -> m c) -> m a -> m b -> m c
+liftJoin2 f ma mb =
+   M.join (lift2 f ma mb)
+
+liftJoin3 :: (Monad m) => (a -> b -> c -> m d) -> m a -> m b -> m c -> m d
+liftJoin3 f ma mb mc =
+   M.join (lift3 f ma mb mc)
+
+liftJoin4 ::
+   (Monad m) =>
+   (a -> b -> c -> d -> m e) ->
+   m a -> m b -> m c -> m d -> m e
+liftJoin4 f ma mb mc md =
+   M.join (lift4 f ma mb mc md)
+
+liftJoin5 ::
+   (Monad m) =>
+   (a -> b -> c -> d -> e -> m f) ->
+   m a -> m b -> m c -> m d -> m e -> m f
+liftJoin5 f ma mb mc md me =
+   M.join (lift5 f ma mb mc md me)
+
 {-
 Add functions with restricted types?
 Shall their element types be monoids?
