@@ -17,6 +17,15 @@ if' :: Bool -> a -> a -> a
 if' True  x _ = x
 if' False _ y = y
 
+{-|
+The same as 'if'', but the name is chosen
+such that it can be used for GHC-7.0's rebindable if-then-else syntax.
+-}
+{-# INLINE ifThenElse #-}
+ifThenElse :: Bool -> a -> a -> a
+ifThenElse = if'
+
+
 {-|
 From a list of expressions choose the one,
 whose condition is true.
