@@ -24,6 +24,8 @@ module Data.List.HT (
    L.switchL,
    L.switchR,
    -- * List processing starting at the end
+   L.dropRev,
+   L.takeRev,
    L.dropWhileRev,
    L.takeWhileRev,
    -- * List processing with Maybe and Either
@@ -51,6 +53,7 @@ module Data.List.HT (
    L.isAscending,
    L.isAscendingLazy,
    L.mapAdjacent,
+   L.mapAdjacent1,
    L.range,
    L.padLeft,
    L.padRight,
