@@ -20,12 +20,22 @@ This function is lazier than the one suggested in the Haskell 98 report.
 It is @inits undefined = [] : undefined@,
 in contrast to @Data.List.inits undefined = undefined@.
 -}
+{-
+suggested in
+<http://www.haskell.org/pipermail/libraries/2014-July/023291.html>
+-}
 inits :: [a] -> [[a]]
-inits xt =
+inits = map reverse . scanl (flip (:)) []
+
+{- |
+As lazy as 'inits' but less efficient because of repeated 'map'.
+-}
+initsLazy :: [a] -> [[a]]
+initsLazy xt =
    [] :
    case xt of
       [] -> []
-      x:xs -> map (x:) (inits xs)
+      x:xs -> map (x:) (initsLazy xs)
 
 {- |
 Suggested implementation in the Haskell 98 report.
@@ -428,6 +438,21 @@ propSwitchR xs =
 
 -- * List processing starting at the end
 
+{- |
+@takeRev n@ is like @reverse . take n . reverse@
+but it is lazy enough to work for infinite lists, too.
+-}
+takeRev :: Int -> [a] -> [a]
+takeRev n xs = Match.drop (drop n xs) xs
+
+{- |
+@dropRev n@ is like @reverse . drop n . reverse@
+but it is lazy enough to work for infinite lists, too.
+-}
+dropRev :: Int -> [a] -> [a]
+dropRev n xs = Match.take (drop n xs) xs
+
+
 {- |
 Remove the longest suffix of elements satisfying p.
 In contrast to @reverse . dropWhile p . reverse@
@@ -921,6 +946,15 @@ in a list with a certain function.
 mapAdjacent :: (a -> a -> b) -> [a] -> [b]
 mapAdjacent f xs = zipWith f xs (tail xs)
 
+{- |
+> mapAdjacent f a0 [(a1,b1), (a2,b2), (a3,b3)]
+> ==
+> [f a0 a1 b1, f a1 a2 b2, f a2 a3 b3]
+-}
+mapAdjacent1 :: (a -> a -> b -> c) -> a -> [(a,b)] -> [c]
+mapAdjacent1 f a xs =
+   zipWith (\a0 (a1,b) -> f a0 a1 b) (a : map fst xs) xs
+
 
 {- |
 Enumerate without Enum context.
