@@ -11,6 +11,24 @@ import Prelude hiding (iterate, )
 
 
 
+takeWhileRev :: (Ord a) => (a -> Bool) -> [a] -> Bool
+takeWhileRev p xs =
+   ListHT.takeWhileRev p xs == reverse (takeWhile p (reverse xs))
+
+dropWhileRev :: (Ord a) => (a -> Bool) -> [a] -> Bool
+dropWhileRev p xs =
+   ListHT.dropWhileRev p xs == reverse (dropWhile p (reverse xs))
+
+
+takeRev :: (Eq a) => Int -> [a] -> Bool
+takeRev n xs =
+   ListHT.takeRev n xs == reverse (take n (reverse xs))
+
+dropRev :: (Eq a) => Int -> [a] -> Bool
+dropRev n xs =
+   ListHT.dropRev n xs == reverse (drop n (reverse xs))
+
+
 sieve :: Eq a => Int -> [a] -> Property
 sieve n x =
    n>0 ==>
@@ -62,6 +80,10 @@ iterate op a =
    in  equalInfLists 1000 [xs, ys, zs]
 
 
+mapAdjacent :: (Num a, Eq a) => a -> [a] -> Bool
+mapAdjacent x xs =
+   ListHT.mapAdjacent subtract (scanl (+) x xs) == xs
+
 
 simple ::
    (Testable test) =>
@@ -71,6 +93,10 @@ simple = quickCheck
 
 tests :: [(String, IO ())]
 tests =
+   ("takeWhileRev",     quickCheck (\a -> takeWhileRev ((a::Integer)>=))) :
+   ("dropWhileRev",     quickCheck (\a -> dropWhileRev ((a::Integer)>=))) :
+   ("takeRev",          simple takeRev) :
+   ("dropRev",          simple dropRev) :
    ("sieve",            simple sieve) :
    ("sliceHorizontal",  simple sliceHorizontal) :
    ("sliceVertical",    simple sliceVertical) :
@@ -78,4 +104,5 @@ tests =
    ("shear",            quickCheck (shear           :: [[Integer]] -> Bool)) :
    ("outerProduct",     quickCheck (outerProduct    :: [Integer] -> [Int] -> Bool)) :
    ("iterate",          quickCheck (iterate (+)     :: Integer -> Bool)) :
+   ("mapAdjacent",      quickCheck (mapAdjacent     :: Integer -> [Integer] -> Bool)) :
    []
